Scanner

Struct Scanner 

pub struct Scanner<'a> { /* private fields */ }
A PostScript scanner that parses Objects from a byte stream.

Implementations

Source

impl<'a> Scanner<'a>

pub fn new(data: &'a [u8]) -> Self

Create a new scanner over the given bytes of a PostScript program.

Examples found in repository?
examples/tokenize.rs (line 27)
10fn main() {
11    let path = match env::args().nth(1) {
12        Some(p) => p,
13        None => {
14            eprintln!("Usage: tokenize <file>");
15            process::exit(1);
16        }
17    };
18
19    let data = match fs::read(&path) {
20        Ok(d) => d,
21        Err(e) => {
22            eprintln!("Error reading {path}: {e}");
23            process::exit(1);
24        }
25    };
26
27    let mut scanner = Scanner::new(&data);
28
29    while !scanner.at_end() {
30        match scanner.parse_object() {
31            Ok(object) => {
32                print_object(&object);
33                println!();
34            }
35            Err(e) => eprintln!("Error: {e}"),
36        }
37    }
38}
pub fn at_end(&mut self) -> bool

Returns true if there are no more objects to parse.

Examples found in repository?
examples/tokenize.rs (line 29)
10fn main() {
11    let path = match env::args().nth(1) {
12        Some(p) => p,
13        None => {
14            eprintln!("Usage: tokenize <file>");
15            process::exit(1);
16        }
17    };
18
19    let data = match fs::read(&path) {
20        Ok(d) => d,
21        Err(e) => {
22            eprintln!("Error reading {path}: {e}");
23            process::exit(1);
24        }
25    };
26
27    let mut scanner = Scanner::new(&data);
28
29    while !scanner.at_end() {
30        match scanner.parse_object() {
31            Ok(object) => {
32                print_object(&object);
33                println!();
34            }
35            Err(e) => eprintln!("Error: {e}"),
36        }
37    }
38}
39
40fn print_object(object: &Object<'_>) {
41    match object {
42        Object::Number(Number::Integer(n)) => print!("Integer({n})"),
43        Object::Number(Number::Real(n)) => print!("Real({n})"),
44        Object::Name(name) => {
45            let kind = if name.is_literal() {
46                "literal"
47            } else {
48                "executable"
49            };
50            let text = name.as_str().unwrap_or("<non-ascii name>");
51            print!("Name({text}, {kind})");
52        }
53        Object::String(s) => {
54            let decoded = s.decode().unwrap_or_else(|_| Vec::new());
55            print!("String({})", String::from_utf8_lossy(&decoded));
56        }
57        Object::Array(arr) => {
58            print!("[");
59            let mut inner = arr.objects();
60            let mut first = true;
61            while !inner.at_end() {
62                if !first {
63                    print!(" ");
64                }
65                first = false;
66                match inner.parse_object() {
67                    Ok(obj) => print_object(&obj),
68                    Err(e) => print!("Error({e})"),
69                }
70            }
71            print!("]");
72        }
73    }
74}
pub fn parse_object(&mut self) -> Result<Object<'a>>

Parse the next object.

Examples found in repository?
examples/tokenize.rs (line 30)
10fn main() {
11    let path = match env::args().nth(1) {
12        Some(p) => p,
13        None => {
14            eprintln!("Usage: tokenize <file>");
15            process::exit(1);
16        }
17    };
18
19    let data = match fs::read(&path) {
20        Ok(d) => d,
21        Err(e) => {
22            eprintln!("Error reading {path}: {e}");
23            process::exit(1);
24        }
25    };
26
27    let mut scanner = Scanner::new(&data);
28
29    while !scanner.at_end() {
30        match scanner.parse_object() {
31            Ok(object) => {
32                print_object(&object);
33                println!();
34            }
35            Err(e) => eprintln!("Error: {e}"),
36        }
37    }
38}
39
40fn print_object(object: &Object<'_>) {
41    match object {
42        Object::Number(Number::Integer(n)) => print!("Integer({n})"),
43        Object::Number(Number::Real(n)) => print!("Real({n})"),
44        Object::Name(name) => {
45            let kind = if name.is_literal() {
46                "literal"
47            } else {
48                "executable"
49            };
50            let text = name.as_str().unwrap_or("<non-ascii name>");
51            print!("Name({text}, {kind})");
52        }
53        Object::String(s) => {
54            let decoded = s.decode().unwrap_or_else(|_| Vec::new());
55            print!("String({})", String::from_utf8_lossy(&decoded));
56        }
57        Object::Array(arr) => {
58            print!("[");
59            let mut inner = arr.objects();
60            let mut first = true;
61            while !inner.at_end() {
62                if !first {
63                    print!(" ");
64                }
65                first = false;
66                match inner.parse_object() {
67                    Ok(obj) => print_object(&obj),
68                    Err(e) => print!("Error({e})"),
69                }
70            }
71            print!("]");
72        }
73    }
74}
pub fn parse_number(&mut self) -> Result<Number>

Parse the next object as a Number.

pub fn parse_name(&mut self) -> Result<Name<'a>>

Parse the next object as a Name.

pub fn parse_string(&mut self) -> Result<String<'a>>

Parse the next object as a String.

pub fn parse_array(&mut self) -> Result<Array<'a>>

Parse the next object as an Array.